
#include <Python.h>
#include "fribidi.h"
#include "fribidi_char_sets_utf8.h"
#include "fribidi_char_sets_cap_rtl.h"

/*
int fribidi_utf8_to_unicode (char *s,
			       int length,
			       /* Output *//*
			       FriBidiChar *us);
*/
/*
fribidi_boolean fribidi_log2vis (	/* input *//*
				    FriBidiChar *str, FriBidiStrIndex len,
				    FriBidiCharType *pbase_dirs,
				    /* output *//*
				    FriBidiChar *visual_str,
				    FriBidiStrIndex *position_L_to_V_list,
				    FriBidiStrIndex *position_V_to_L_list,
				    FriBidiLevel *embedding_level_list);
*/

static PyObject *ErrorObject;

static int use_cap_rtl = 0;

/* ----------------------------------------------------- */

static char _pyf_log2vis__doc__[] =
"takes utf-8 encoded string and an optional direction override parameter\n" \
"0 means LTR, 1 means RTL, anything else means no override"
;

static PyObject *
_pyf_log2vis(self, args)
	PyObject *self;	/* Not used */
	PyObject *args;
{
    char *instr;
    int incount;
    FriBidiChar *uinstr;
    FriBidiChar *uoutstr;
    char *outstr;
    PyObject *res;
    int len;
    FriBidiCharType base_dir = FRIBIDI_TYPE_ON;
    fribidi_boolean log2vis_result;
    int py_base_dir = 2;

	if (!PyArg_ParseTuple(args, "s#|i", &instr, &incount, &py_base_dir))
		return NULL;
    if (py_base_dir==0) {
        base_dir = FRIBIDI_TYPE_LTR;
    }
    if (py_base_dir==1) {
        base_dir = FRIBIDI_TYPE_RTL;
    }

    //printf("input len = %d\n", incount);
    if((uinstr = (FriBidiChar *) malloc((incount+1)*sizeof(FriBidiChar))) == NULL){
        PyErr_NoMemory();
        return NULL;
    }
    if((uoutstr = (FriBidiChar *) malloc((incount+1)*sizeof(FriBidiChar))) == NULL){
        PyErr_NoMemory();
        free(uinstr);
        return NULL;
    }
    if((outstr = (char *) malloc((incount+1)*sizeof(FriBidiChar))) == NULL){
        PyErr_NoMemory();
        free(uinstr);
        free(uoutstr);
        return NULL;
    }

    if(use_cap_rtl) {
        len = fribidi_cap_rtl_to_unicode(instr, incount, uinstr);
    }
    else {
        len = fribidi_utf8_to_unicode(instr, incount, uinstr);
    }
    //printf("utf8_to_unicode len = %d\n", len);

    log2vis_result = fribidi_log2vis(uinstr, len, &base_dir, uoutstr, NULL, NULL, NULL);
    //printf("back.");

    if(use_cap_rtl) {
        len = fribidi_unicode_to_cap_rtl(uoutstr, len, outstr);
    }
    else {
        len = fribidi_unicode_to_utf8(uoutstr, len, outstr);
    }
    //printf("unicode_to_utf8 len = %d\n", len);

    res = Py_BuildValue("s", outstr);

    free(uinstr);
    free(uoutstr);
    free(outstr);

    return res;
	//Py_INCREF(Py_None);
	//return Py_None;
}


static PyObject *
_pyf_enable_cap_rtl(self, args)
	PyObject *self;	/* Not used */
	PyObject *args;
{
    use_cap_rtl = 1;
    fribidi_char_set_enter_cap_rtl();
	Py_INCREF(Py_None);
	return Py_None;
}
/* List of methods defined in the module */

static struct PyMethodDef _pyf_methods[] = {
	{"log2vis",	(PyCFunction)_pyf_log2vis,	METH_VARARGS,	_pyf_log2vis__doc__},
	{"enable_cap_rtl",	(PyCFunction)_pyf_enable_cap_rtl,	METH_VARARGS,	NULL},
 
	{NULL,	 (PyCFunction)NULL, 0, NULL}		/* sentinel */
};


/* Initialization function for the module (*must* be called init_pyfribidi) */

static char _pyfribidi_module_documentation[] = 
""
;

void
init_pyfribidi()
{
	PyObject *m;

	/* Create the module and add the functions */
	m = Py_InitModule4("_pyfribidi", _pyf_methods,
		_pyfribidi_module_documentation,
		(PyObject*)NULL,PYTHON_API_VERSION);

	/* Add some symbolic constants to the module */
/*
	d = PyModule_GetDict(m);
	ErrorObject = PyString_FromString("_pyfribidi.error");
	PyDict_SetItemString(d, "error", ErrorObject);
*/
	/* XXXX Add constants here */
	
	/* Check for errors */
	if (PyErr_Occurred())
		Py_FatalError("can't initialize module _pyfribidi");
}

